@@ -316,10 +316,8 @@ typedef const char* (*csync_checksum_hook) (
  * @brief Allocate a csync context.
  *
  * @param csync  The context variable to allocate.
- *
- * @return  0 on success, less than 0 if an error occured.
  */
-int csync_create(CSYNC **csync, const char *local, const char *remote);
+void csync_create(CSYNC **csync, const char *local, const char *remote);
 
 /**
  * @brief Initialize the file synchronizer.
@@ -327,17 +325,15 @@ int csync_create(CSYNC **csync, const char *local, const char *remote);
  * This function loads the configuration
  *
  * @param ctx  The context to initialize.
- *
- * @return  0 on success, less than 0 if an error occured.
  */
-int csync_init(CSYNC *ctx);
+void csync_init(CSYNC *ctx);
 
 /**
  * @brief Update detection
  *
  * @param ctx  The context to run the update detection on.
  *
- * @return  0 on success, less than 0 if an error occured.
+ * @return  0 on success, less than 0 if an error occurred.
  */
 int csync_update(CSYNC *ctx);
 
@@ -346,7 +342,7 @@ int csync_update(CSYNC *ctx);
  *
  * @param ctx  The context to run the reconciliation on.
  *
- * @return  0 on success, less than 0 if an error occured.
+ * @return  0 on success, less than 0 if an error occurred.
  */
 int csync_reconcile(CSYNC *ctx);
 
@@ -355,7 +351,7 @@ int csync_reconcile(CSYNC *ctx);
  *
  * @param ctx  The context to commit.
  *
- * @return  0 on success, less than 0 if an error occured.
+ * @return  0 on success, less than 0 if an error occurred.
  */
 int csync_commit(CSYNC *ctx);
 
@@ -366,35 +362,17 @@ int csync_commit(CSYNC *ctx);
  *
  * @param ctx  The context to destroy.
  *
- * @return  0 on success, less than 0 if an error occured.
+ * @return  0 on success, less than 0 if an error occurred.
  */
 int csync_destroy(CSYNC *ctx);
 
-/**
- * @brief Add an additional exclude list.
- *
- * @param ctx           The context to add the exclude list.
- *
- * @param path          The path pointing to the file.
- *
- * @return              0 on success, less than 0 if an error occured.
- */
-int csync_add_exclude_list(CSYNC *ctx, const char *path);
-
-/**
- * @brief Removes all items imported from exclude lists.
- *
- * @param ctx           The context to add the exclude list.
- */
-void csync_clear_exclude_list(CSYNC *ctx);
-
 /**
  * @brief Get the userdata saved in the context.
  *
  * @param ctx           The csync context.
  *
  * @return              The userdata saved in the context, NULL if an error
- *                      occured.
+ *                      occurred.
  */
 void *csync_get_userdata(CSYNC *ctx);
 
@@ -406,7 +384,7 @@ void *csync_get_userdata(CSYNC *ctx);
  *
  * @param userdata      The userdata to be stored in the context.
  *
- * @return              0 on success, less than 0 if an error occured.
+ * @return              0 on success, less than 0 if an error occurred.
  */
 int csync_set_userdata(CSYNC *ctx, void *userdata);
 
@@ -416,7 +394,7 @@ int csync_set_userdata(CSYNC *ctx, void *userdata);
  * @param ctx           The csync context.
  *
  * @return              The authentication callback set or NULL if an error
- *                      occured.
+ *                      occurred.
  */
 csync_auth_callback csync_get_auth_callback(CSYNC *ctx);
 
@@ -427,7 +405,7 @@ csync_auth_callback csync_get_auth_callback(CSYNC *ctx);
  *
  * @param cb            The authentication callback.
  *
- * @return              0 on success, less than 0 if an error occured.
+ * @return              0 on success, less than 0 if an error occurred.
  */
 int csync_set_auth_callback(CSYNC *ctx, csync_auth_callback cb);
 
@@ -436,7 +414,7 @@ int csync_set_auth_callback(CSYNC *ctx, csync_auth_callback cb);
  *
  * @param[in]  level  The log verbosity.
  *
- * @return 0 on success, < 0 if an error occured.
+ * @return 0 on success, < 0 if an error occurred.
  */
 int csync_set_log_level(int level);
 
@@ -451,7 +429,7 @@ int csync_get_log_level(void);
  * @brief Get the logging callback set.
  *
  * @return              The logging callback set or NULL if an error
- *                      occured.
+ *                      occurred.
  */
 csync_log_callback csync_get_log_callback(void);
 
@@ -460,7 +438,7 @@ csync_log_callback csync_get_log_callback(void);
  *
  * @param cb            The logging callback.
  *
- * @return              0 on success, less than 0 if an error occured.
+ * @return              0 on success, less than 0 if an error occurred.
  */
 int csync_set_log_callback(csync_log_callback cb);
 
@@ -476,7 +454,7 @@ void *csync_get_log_userdata(void);
  *
  * @param[in]  data     The userdata to set.
  *
- * @return              0 on success, less than 0 if an error occured.
+ * @return              0 on success, less than 0 if an error occurred.
  */
 int csync_set_log_userdata(void *data);
 
@@ -495,7 +473,7 @@ typedef int csync_treewalk_visit_func(TREE_WALK_FILE* ,void*);
  * @param visitor       A callback function to handle the file info.
  * @param filter        A filter, built from or'ed csync_instructions_e
  *
- * @return              0 on success, less than 0 if an error occured.
+ * @return              0 on success, less than 0 if an error occurred.
  */
 int csync_walk_local_tree(CSYNC *ctx, csync_treewalk_visit_func *visitor, int filter);
 
@@ -506,7 +484,7 @@ int csync_walk_local_tree(CSYNC *ctx, csync_treewalk_visit_func *visitor, int fi
  * @param visitor       A callback function to handle the file info.
  * @param filter        A filter, built from and'ed csync_instructions_e
  *
- * @return              0 on success, less than 0 if an error occured.
+ * @return              0 on success, less than 0 if an error occurred.
  */
 int csync_walk_remote_tree(CSYNC *ctx, csync_treewalk_visit_func *visitor, int filter);
 
