@@ -37,8 +37,7 @@ static void setup(void **state) {
     rc = system("mkdir -p /tmp/check_csync2");
     assert_int_equal(rc, 0);
 
-    rc = csync_create(&csync, "/tmp/check_csync1", "/tmp/check_csync2");
-    assert_int_equal(rc, 0);
+    csync_create(&csync, "/tmp/check_csync1", "/tmp/check_csync2");
 
     *state = csync;
 }
