@@ -97,8 +97,7 @@ static void setup_testenv(void **state) {
     statevar *mystate = malloc( sizeof(statevar) );
     mystate->result = NULL;
 
-    rc = csync_create(&(mystate->csync), "/tmp/csync1", "/tmp/csync2");
-    assert_int_equal(rc, 0);
+    csync_create(&(mystate->csync), "/tmp/csync1", "/tmp/csync2");
 
     mystate->csync->replica = LOCAL_REPLICA;
 
