@@ -31,6 +31,10 @@ class SelectiveSyncTreeView;
 
 class ownCloudInfo;
 
+/**
+ * @brief The FormatWarningsWizardPage class
+ * @ingroup gui
+ */
 class FormatWarningsWizardPage : public QWizardPage {
     Q_OBJECT
 protected:
@@ -38,7 +42,8 @@ protected:
 };
 
 /**
- * page to ask for the local source folder
+ * @brief Page to ask for the local source folder
+ * @ingroup gui
  */
 class FolderWizardLocalPath : public FormatWarningsWizardPage
 {
@@ -62,7 +67,8 @@ private:
 
 
 /**
- * page to ask for the target folder
+ * @brief page to ask for the target folder
+ * @ingroup gui
  */
 
 class FolderWizardRemotePath : public FormatWarningsWizardPage
@@ -83,7 +89,8 @@ protected slots:
     void slotAddRemoteFolder();
     void slotCreateRemoteFolder(const QString&);
     void slotCreateRemoteFolderFinished(QNetworkReply::NetworkError error);
-    void slotHandleNetworkError(QNetworkReply*);
+    void slotHandleMkdirNetworkError(QNetworkReply*);
+    void slotHandleLsColNetworkError(QNetworkReply*);
     void slotUpdateDirectories(const QStringList&);
     void slotRefreshFolders();
     void slotItemExpanded(QTreeWidgetItem*);
@@ -91,7 +98,9 @@ protected slots:
     void slotFolderEntryEdited(const QString& text);
     void slotLsColFolderEntry();
     void slotTypedPathFound(const QStringList& subpaths);
+    void slotTypedPathError(QNetworkReply* reply);
 private:
+    LsColJob* runLsColJob(const QString& path);
     void recursiveInsert(QTreeWidgetItem *parent, QStringList pathTrail, QString path);
     bool selectByPath(QString path);
     Ui_FolderWizardTargetPage _ui;
@@ -100,7 +109,10 @@ private:
     QTimer _lscolTimer;
 };
 
-
+/**
+ * @brief The FolderWizardSelectiveSync class
+ * @ingroup gui
+ */
 class FolderWizardSelectiveSync : public QWizardPage
 {
     Q_OBJECT
@@ -118,9 +130,9 @@ private:
 
 };
 
-
 /**
- *
+ * @brief The FolderWizard class
+ * @ingroup gui
  */
 class FolderWizard : public QWizard
 {
