@@ -23,6 +23,7 @@
 #include "utility.h"
 #include "theme.h"
 #include "syncjournalfilerecord.h"
+#include "syncengine.h"
 #include "syncfileitem.h"
 #include "filesystem.h"
 #include "version.h"
@@ -55,6 +56,13 @@
 // The second number should be changed when there are new features.
 #define MIRALL_SOCKET_API_VERSION "1.0"
 
+static inline QString removeTrailingSlash(QString path)
+{
+    Q_ASSERT(path.endsWith(QLatin1Char('/')));
+    path.truncate(path.length()-1);
+    return path;
+}
+
 namespace OCC {
 
 #define DEBUG qDebug() << "SocketApi: "
@@ -66,7 +74,8 @@ SocketApi::SocketApi(QObject* parent)
 
     if (Utility::isWindows()) {
         socketPath = QLatin1String("\\\\.\\pipe\\")
-        + QLatin1String("ownCloud");
+                + QLatin1String("ownCloud-")
+                + QString::fromLocal8Bit(qgetenv("USERNAME"));
         // TODO: once the windows extension supports multiple
         // client connections, switch back to the theme name
         // See issue #2388
@@ -113,10 +122,6 @@ SocketApi::SocketApi(QObject* parent)
 
     // folder watcher
     connect(FolderMan::instance(), SIGNAL(folderSyncStateChange(Folder*)), this, SLOT(slotUpdateFolderView(Folder*)));
-    connect(ProgressDispatcher::instance(), SIGNAL(itemCompleted(QString, const SyncFileItem &, const PropagatorJob &)),
-            SLOT(slotItemCompleted(QString, const SyncFileItem &)));
-    connect(ProgressDispatcher::instance(), SIGNAL(syncItemDiscovered(QString, const SyncFileItem &)),
-            this, SLOT(slotSyncItemDiscovered(QString, const SyncFileItem &)));
 }
 
 SocketApi::~SocketApi()
@@ -143,8 +148,10 @@ void SocketApi::slotNewConnection()
     _listeners.append(socket);
 
     foreach( Folder *f, FolderMan::instance()->map() ) {
-        QString message = buildRegisterPathMessage(f->path());
-        sendMessage(socket, message);
+        if (f->canSync()) {
+            QString message = buildRegisterPathMessage(removeTrailingSlash(f->path()));
+            sendMessage(socket, message);            
+        }
     }
 }
 
@@ -183,36 +190,31 @@ void SocketApi::slotReadSocket()
 
 void SocketApi::slotRegisterPath( const QString& alias )
 {
+    // Make sure not to register twice to each connected client
+    if (_registeredAliases.contains(alias))
+        return;
+
     Folder *f = FolderMan::instance()->folder(alias);
     if (f) {
-        QString message = buildRegisterPathMessage(f->path());
+        QString message = buildRegisterPathMessage(removeTrailingSlash(f->path()));
         foreach(QIODevice *socket, _listeners) {
             sendMessage(socket, message);
         }
     }
+
+    _registeredAliases.insert(alias);
 }
 
 void SocketApi::slotUnregisterPath( const QString& alias )
 {
-    Folder *f = FolderMan::instance()->folder(alias);
-    if (f) {
-        broadcastMessage(QLatin1String("UNREGISTER_PATH"), f->path(), QString::null, true );
+    if (!_registeredAliases.contains(alias))
+        return;
 
-        if( _dbQueries.contains(f)) {
-            auto h = _dbQueries[f];
-            if( h ) {
-                h->finish();
-            }
-            _dbQueries.remove(f);
-        }
-        if( _openDbs.contains(f) ) {
-            auto db = _openDbs[f];
-            if( db ) {
-                db->close();
-            }
-            _openDbs.remove(f);
-        }
-    }
+    Folder *f = FolderMan::instance()->folder(alias);
+    if (f)
+        broadcastMessage(QLatin1String("UNREGISTER_PATH"), removeTrailingSlash(f->path()), QString::null, true );
+
+    _registeredAliases.remove(alias);
 }
 
 void SocketApi::slotUpdateFolderView(Folder *f)
@@ -230,58 +232,22 @@ void SocketApi::slotUpdateFolderView(Folder *f)
                 f->syncResult().status() == SyncResult::Error   ||
                 f->syncResult().status() == SyncResult::SetupError ) {
 
-            broadcastMessage(QLatin1String("STATUS"), f->path() ,
-                             this->fileStatus(f, "").toSocketAPIString());
+            QString rootPath = removeTrailingSlash(f->path());
+            broadcastMessage(QLatin1String("STATUS"), rootPath,
+                             f->syncEngine().syncFileStatusTracker().fileStatus("").toSocketAPIString());
 
-            broadcastMessage(QLatin1String("UPDATE_VIEW"), f->path() );
+            broadcastMessage(QLatin1String("UPDATE_VIEW"), rootPath);
         } else {
             qDebug() << "Not sending UPDATE_VIEW for" << f->alias() << "because status() is" << f->syncResult().status();
         }
     }
 }
 
-void SocketApi::slotItemCompleted(const QString &folder, const SyncFileItem &item)
+void SocketApi::slotFileStatusChanged(const QString& systemFileName, SyncFileStatus fileStatus)
 {
-    if (_listeners.isEmpty()) {
-        return;
-    }
-
-    Folder *f = FolderMan::instance()->folder(folder);
-    if (!f) {
-        return;
-    }
-
-    auto status = this->fileStatus(f, item.destination());
-    const QString path = f->path() + item.destination();
-    broadcastMessage(QLatin1String("STATUS"), path, status.toSocketAPIString());
+    broadcastMessage(QLatin1String("STATUS"), systemFileName, fileStatus.toSocketAPIString());
 }
 
-void SocketApi::slotSyncItemDiscovered(const QString &folder, const SyncFileItem &item)
-{
-    if (_listeners.isEmpty()) {
-        return;
-    }
-
-    Folder *f = FolderMan::instance()->folder(folder);
-    if (!f) {
-        return;
-    }
-
-    QString path = f->path() + item.destination();
-
-    // the trailing slash for directories must be appended as the filenames coming in
-    // from the plugins have that too. Otherwise the matching entry item is not found
-    // in the plugin.
-    if( item._type == SyncFileItem::Type::Directory ) {
-        path += QLatin1Char('/');
-    }
-
-    const QString command = QLatin1String("SYNC");
-    broadcastMessage(QLatin1String("STATUS"), path, command);
-}
-
-
-
 void SocketApi::sendMessage(QIODevice *socket, const QString& message, bool doWait)
 {
     DEBUG << "Sending message: " << message;
@@ -342,11 +308,14 @@ void SocketApi::command_RETRIEVE_FILE_STATUS(const QString& argument, QIODevice*
     Folder* syncFolder = FolderMan::instance()->folderForPath( argument );
     if (!syncFolder) {
         // this can happen in offline mode e.g.: nothing to worry about
-        DEBUG << "folder offline or not watched:" << argument;
         statusString = QLatin1String("NOP");
     } else {
-        const QString file = QDir::cleanPath(argument).mid(syncFolder->cleanPath().length()+1);
-        SyncFileStatus fileStatus = this->fileStatus(syncFolder, file);
+        QString relativePath = QDir::cleanPath(argument).mid(syncFolder->cleanPath().length()+1);
+        if( relativePath.endsWith(QLatin1Char('/')) ) {
+            relativePath.truncate(relativePath.length()-1);
+            qWarning() << "Removed trailing slash for directory: " << relativePath << "Status pushes won't have one.";
+        }
+        SyncFileStatus fileStatus = syncFolder->syncEngine().syncFileStatusTracker().fileStatus(relativePath);
 
         statusString = fileStatus.toSocketAPIString();
     }
@@ -364,6 +333,8 @@ void SocketApi::command_SHARE(const QString& localFile, QIODevice* socket)
 
     qDebug() << Q_FUNC_INFO << localFile;
 
+    auto theme = Theme::instance();
+
     Folder *shareFolder = FolderMan::instance()->folderForPath(localFile);
     if (!shareFolder) {
         const QString message = QLatin1String("SHARE:NOP:")+QDir::toNativeSeparators(localFile);
@@ -373,20 +344,24 @@ void SocketApi::command_SHARE(const QString& localFile, QIODevice* socket)
         const QString message = QLatin1String("SHARE:NOTCONNECTED:")+QDir::toNativeSeparators(localFile);
         // if the folder isn't connected, don't open the share dialog
         sendMessage(socket, message);
+    } else if (!theme->linkSharing() && (
+                 !theme->userGroupSharing() ||
+                 shareFolder->accountState()->account()->serverVersionInt() < ((8 << 16) + (2 << 8)))) {
+        const QString message = QLatin1String("SHARE:NOP:")+QDir::toNativeSeparators(localFile);
+        sendMessage(socket, message);
     } else {
         const QString localFileClean = QDir::cleanPath(localFile);
         const QString file = localFileClean.mid(shareFolder->cleanPath().length()+1);
-        SyncFileStatus fileStatus = this->fileStatus(shareFolder, file);
+        SyncFileStatus fileStatus = shareFolder->syncEngine().syncFileStatusTracker().fileStatus(file);
 
         // Verify the file is on the server (to our knowledge of course)
-        if (fileStatus.tag() != SyncFileStatus::STATUS_UPTODATE &&
-            fileStatus.tag() != SyncFileStatus::STATUS_UPDATED) {
+        if (fileStatus.tag() != SyncFileStatus::StatusUpToDate) {
             const QString message = QLatin1String("SHARE:NOTSYNCED:")+QDir::toNativeSeparators(localFile);
             sendMessage(socket, message);
             return;
         }
 
-        const QString remotePath = shareFolder->remotePath() + QLatin1Char('/') + file;
+        const QString remotePath = QDir(shareFolder->remotePath()).filePath(file);
 
         // Can't share root folder
         if (remotePath == "/") {
@@ -395,7 +370,7 @@ void SocketApi::command_SHARE(const QString& localFile, QIODevice* socket)
             return;
         }
 
-        SyncJournalFileRecord rec = dbFileRecord_capi(shareFolder, localFileClean);
+        SyncJournalFileRecord rec = shareFolder->journalDb()->getFileRecord(localFileClean);
 
         bool allowReshare = true; // lets assume the good
         if( rec.isValid() ) {
@@ -432,11 +407,10 @@ void SocketApi::command_SHARE_STATUS(const QString &localFile, QIODevice *socket
         sendMessage(socket, message);
     } else {
         const QString file = QDir::cleanPath(localFile).mid(shareFolder->cleanPath().length()+1);
-        SyncFileStatus fileStatus = this->fileStatus(shareFolder, file);
+        SyncFileStatus fileStatus = shareFolder->syncEngine().syncFileStatusTracker().fileStatus(file);
 
         // Verify the file is on the server (to our knowledge of course)
-        if (fileStatus.tag() != SyncFileStatus::STATUS_UPTODATE &&
-            fileStatus.tag() != SyncFileStatus::STATUS_UPDATED) {
+        if (fileStatus.tag() != SyncFileStatus::StatusUpToDate) {
             const QString message = QLatin1String("SHARE_STATUS:NOTSYNCED:")+QDir::toNativeSeparators(localFile);
             sendMessage(socket, message);
             return;
@@ -448,14 +422,28 @@ void SocketApi::command_SHARE_STATUS(const QString &localFile, QIODevice *socket
             const QString message = QLatin1String("SHARE_STATUS:DISABLED:")+QDir::toNativeSeparators(localFile);
             sendMessage(socket, message);
         } else {
-            QString available = "USER,GROUP";
+            auto theme = Theme::instance();
+            QString available;
 
-            if (capabilities.sharePublicLink()) {
-                available += ",LINK";
+            if (theme->userGroupSharing()) {
+                available = "USER,GROUP";
             }
 
-            const QString message = QLatin1String("SHARE_STATUS:") + available + ":" + QDir::toNativeSeparators(localFile);
-            sendMessage(socket, message);
+            if (theme->linkSharing() && capabilities.sharePublicLink()) {
+                if (available.isEmpty()) {
+                    available = "LINK";
+                } else {
+                    available += ",LINK";
+                }
+            }
+
+            if (available.isEmpty()) {
+                const QString message = QLatin1String("SHARE_STATUS:DISABLED") + ":" + QDir::toNativeSeparators(localFile);
+                sendMessage(socket, message);
+            } else {
+                const QString message = QLatin1String("SHARE_STATUS:") + available + ":" + QDir::toNativeSeparators(localFile);
+                sendMessage(socket, message);
+            }
         }
     }
 }
@@ -473,227 +461,4 @@ QString SocketApi::buildRegisterPathMessage(const QString& path)
     return message;
 }
 
-SqlQuery* SocketApi::getSqlQuery( Folder *folder )
-{
-    if( !folder ) {
-        return 0;
-    }
-
-    if( _dbQueries.contains(folder) ) {
-        return _dbQueries[folder].data();
-    }
-
-    /* No valid sql query object yet for this folder */
-    int rc;
-    const QString sql("SELECT inode, modtime, type, md5, fileid, remotePerm FROM "
-                      "metadata WHERE phash=?1");
-    QString dbFileName = folder->journalDb()->databaseFilePath();
-
-    QFileInfo fi(dbFileName);
-    if( fi.exists() ) {
-        auto db = QSharedPointer<SqlDatabase>::create();
-
-        if( db && db->openReadOnly(dbFileName) ) {
-            _openDbs.insert(folder, db);
-
-            QSharedPointer<SqlQuery> query(new SqlQuery(*db));
-            rc = query->prepare(sql);
-
-            if( rc != SQLITE_OK ) {
-                qDebug() << "Unable to prepare the query statement:" << rc;
-                return 0; // do not insert into hash
-            }
-            _dbQueries.insert( folder, query);
-            return query.data();
-        } else {
-            qDebug() << "Unable to open db" << dbFileName;
-        }
-    } else {
-        qDebug() << Q_FUNC_INFO << "Journal to query does not yet exist.";
-    }
-    return 0;
-}
-
-SyncJournalFileRecord SocketApi::dbFileRecord_capi( Folder *folder, QString fileName )
-{
-    if( !(folder && folder->journalDb()) ) {
-        return SyncJournalFileRecord();
-    }
-
-    if( fileName.startsWith( folder->path() )) {
-        fileName.remove(0, folder->path().length());
-    }
-
-    // remove trailing slash
-    if( fileName.endsWith( QLatin1Char('/') ) ) {
-        fileName.truncate(fileName.length()-1);
-    }
-    SqlQuery *query = getSqlQuery(folder);
-    SyncJournalFileRecord rec;
-
-    if( query ) {
-        qlonglong phash = SyncJournalDb::getPHash( fileName );
-        query->bindValue(1, phash);
-        // int column_count = sqlite3_column_count(stmt);
-
-        if (query->next()) {
-            rec._path    = fileName;
-            rec._inode   = query->int64Value(0);
-            rec._modtime = Utility::qDateTimeFromTime_t( query->int64Value(1));
-            rec._type    = query->intValue(2);
-            rec._etag    = query->baValue(3);
-            rec._fileId  = query->baValue(4);
-            rec._remotePerm = query->baValue(5);
-        }
-        query->reset();
-    }
-    return rec;
-}
-
-/**
- * Get status about a single file.
- */
-SyncFileStatus SocketApi::fileStatus(Folder *folder, const QString& systemFileName)
-{
-    QString file = folder->path();
-    QString fileName = systemFileName.normalized(QString::NormalizationForm_C);
-    QString fileNameSlash = fileName;
-
-    if(fileName != QLatin1String("/") && !fileName.isEmpty()) {
-        file += fileName;
-    }
-
-    if( fileName.endsWith(QLatin1Char('/')) ) {
-        fileName.truncate(fileName.length()-1);
-        qDebug() << "Removed trailing slash: " << fileName;
-    } else {
-        fileNameSlash += QLatin1Char('/');
-    }
-
-    const QFileInfo fi(file);
-    if( !FileSystem::fileExists(file, fi) ) {
-        qDebug() << "OO File " << file << " is not existing";
-        return SyncFileStatus(SyncFileStatus::STATUS_STAT_ERROR);
-    }
-
-    // file is ignored?
-    // Qt considers .lnk files symlinks on Windows so we need to work
-    // around that here.
-    if( fi.isSymLink()
-#ifdef Q_OS_WIN
-            && fi.suffix() != "lnk"
-#endif
-            ) {
-        return SyncFileStatus(SyncFileStatus::STATUS_IGNORE);
-    }
-
-    csync_ftw_type_e type = CSYNC_FTW_TYPE_FILE;
-    if( fi.isDir() ) {
-        type = CSYNC_FTW_TYPE_DIR;
-    }
-
-    // Is it excluded?
-    if( folder->isFileExcludedRelative(fileName) ) {
-        return SyncFileStatus(SyncFileStatus::STATUS_IGNORE);
-    }
-
-    // Error if it is in the selective sync blacklist
-    foreach(const auto &s, folder->journalDb()->getSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList)) {
-        if (fileNameSlash.startsWith(s)) {
-            return SyncFileStatus(SyncFileStatus::STATUS_ERROR);
-        }
-    }
-
-    SyncFileStatus status(SyncFileStatus::STATUS_NONE);
-    SyncJournalFileRecord rec = dbFileRecord_capi(folder, fileName );
-
-    if (folder->estimateState(fileName, type, &status)) {
-        qDebug() << "Folder estimated status for" << fileName << "to" << status.toSocketAPIString();
-    } else if (fileName == "") {
-        // sync folder itself
-        switch (folder->syncResult().status()) {
-        case SyncResult::Undefined:
-        case SyncResult::NotYetStarted:
-        case SyncResult::SyncPrepare:
-        case SyncResult::SyncRunning:
-            status.set(SyncFileStatus::STATUS_EVAL);
-            return status;
-
-        case SyncResult::Success:
-        case SyncResult::Problem:
-            status.set(SyncFileStatus::STATUS_UPTODATE);
-            return status;
-
-        case SyncResult::Error:
-        case SyncResult::SetupError:
-        case SyncResult::SyncAbortRequested:
-            status.set(SyncFileStatus::STATUS_ERROR);
-            return status;
-
-        case SyncResult::Paused:
-            status.set(SyncFileStatus::STATUS_IGNORE);
-            return status;
-        }
-    } else if (type == CSYNC_FTW_TYPE_DIR) {
-        if (rec.isValid()) {
-            status.set(SyncFileStatus::STATUS_UPTODATE);
-        } else {
-            qDebug() << "Could not determine state for folder" << fileName << "will set STATUS_NEW";
-            status.set(SyncFileStatus::STATUS_NEW);
-        }
-    } else if (type == CSYNC_FTW_TYPE_FILE) {
-        if (rec.isValid()) {
-            if( FileSystem::getModTime(fi.absoluteFilePath()) == Utility::qDateTimeToTime_t(rec._modtime) ) {
-                status.set(SyncFileStatus::STATUS_UPTODATE);
-            } else {
-                if (rec._remotePerm.isNull() || rec._remotePerm.contains("W") ) {
-                    status.set(SyncFileStatus::STATUS_EVAL);
-                } else {
-                    status.set(SyncFileStatus::STATUS_ERROR);
-                }
-            }
-        } else {
-            qDebug() << "Could not determine state for file" << fileName << "will set STATUS_NEW";
-            status.set(SyncFileStatus::STATUS_NEW);
-        }
-    }
-
-    if (rec.isValid()) {
-        if (rec._remotePerm.isNull()) {
-            // probably owncloud 6, that does not have permissions flag yet.
-            QString url = folder->remoteUrl().toString() + fileName;
-            if (url.contains( folder->accountState()->account()->davPath() + QLatin1String("Shared/") )) {
-                status.setSharedWithMe(true);
-            }
-        } else if (rec._remotePerm.contains("S")) {
-            status.setSharedWithMe(true);
-        }
-    }
-    if (status.tag() == SyncFileStatus::STATUS_NEW) {
-        // check the parent folder if it is shared and if it is allowed to create a file/dir within
-        QDir d( fi.path() );
-        auto parentPath = d.path();
-        auto dirRec = dbFileRecord_capi(folder, parentPath);
-        bool isDir = type == CSYNC_FTW_TYPE_DIR;
-        while( !d.isRoot() && !(d.exists() && dirRec.isValid()) ) {
-            d.cdUp(); // returns true if the dir exists.
-
-            parentPath = d.path();
-            // cut the folder path
-            dirRec = dbFileRecord_capi(folder, parentPath);
-
-            isDir = true;
-        }
-        if( dirRec.isValid() && !dirRec._remotePerm.isNull()) {
-            if( (isDir && !dirRec._remotePerm.contains("K"))
-                    || (!isDir && !dirRec._remotePerm.contains("C")) ) {
-                status.set(SyncFileStatus::STATUS_ERROR);
-            }
-        }
-    }
-    return status;
-}
-
-
 } // namespace OCC
-
