@@ -300,8 +300,7 @@ void SqlQuery::bindValue(int pos, const QVariant& value)
                 res = sqlite3_bind_text16(_stmt, pos, str->utf16(),
                                           (str->size()) * sizeof(QChar), SQLITE_TRANSIENT);
             } else {
-                // unbound value create a null entry.
-                res = SQLITE_OK;
+                res = sqlite3_bind_null(_stmt, pos);
             }
             break; }
         default: {
@@ -349,6 +348,11 @@ QString SqlQuery::error() const
     return _error;
 }
 
+int SqlQuery::errorId() const
+{
+    return _errId;
+}
+
 QString SqlQuery::lastQuery() const
 {
     return _sql;
@@ -365,9 +369,10 @@ void SqlQuery::finish()
     _stmt = 0;
 }
 
-void SqlQuery::reset()
+void SqlQuery::reset_and_clear_bindings()
 {
     SQLITE_DO(sqlite3_reset(_stmt));
+    SQLITE_DO(sqlite3_clear_bindings(_stmt));
 }
 
 } // namespace OCC
